@@ -3,7 +3,7 @@ use strict;
 use warnings;
 package App::Spec::Argument;
 
-our $VERSION = '0.005_001'; # TRIAL VERSION
+our $VERSION = '0.008_001'; # TRIAL VERSION
 
 use Moo;
 
@@ -58,13 +58,12 @@ sub from_dsl {
 
     my $name;
     my $type = "flag";
-    my $multiple = 0;
-    my $mapping = 0;
-    my $required = 0;
+    my $multiple;
     $dsl =~ s/^\s+//;
 
     if ($dsl =~ s/^\+//) {
-        $required = 1;
+        my $required = 1;
+        $hash{required} = $required;
     }
 
     if ($dsl =~ s/^ ($name_re) //x) {
@@ -87,12 +86,14 @@ sub from_dsl {
     if ($dsl =~ s/^=//) {
         # not a flag, default string
         $type = "string";
-        # TODO support all of Getopt::Long types
-        if ($dsl =~ s/^([is])//) {
+        if ($dsl =~ s/^([isf])//) {
             $getopt_type = $1;
             if ($getopt_type eq "i") {
                 $type = "integer";
             }
+            elsif ($getopt_type eq "f") {
+                $type = "float";
+            }
             elsif ($getopt_type eq "s") {
             }
             else {
@@ -104,13 +105,14 @@ sub from_dsl {
     if ($type eq 'flag' and $dsl =~ s/^\+//) {
         # incremental flag
         $multiple = 1;
+        $hash{multiple} = 1;
     }
     elsif ($type eq 'string' and $dsl =~ s/^\@//) {
-        $multiple = 1;
+        $hash{multiple} = 1;
     }
     elsif ($type eq 'string' and $dsl =~ s/^\%//) {
-        $multiple = 1;
-        $mapping = 1;
+        $hash{multiple} = 1;
+        $hash{mapping} = 1;
     }
 
     $dsl =~ s/^\s+//;
@@ -137,9 +139,6 @@ sub from_dsl {
     }
 
     $hash{type} = $type;
-    $hash{multiple} = $multiple;
-    $hash{mapping} = $mapping;
-    $hash{required} = $required;
     return %hash;
 }
 
@@ -182,6 +181,8 @@ START INLINE t/data/12.dsl.yaml
       - spec: +req                --Some required flag
       - spec: number=i            --integer option
       - spec: number2|n= +integer --integer option
+      - spec: fnumber=f           --float option
+      - spec: fnumber2|f= +float  --float option
       - spec: date|d=s =today
       - spec: items=s@            --multi option
       - spec: set=s%              --multiple key=value pairs
@@ -212,6 +213,13 @@ START INLINE t/data/12.dsl.yaml
         summary: integer option
         type: integer
         aliases: ["n"]
+      - name: fnumber
+        summary: float option
+        type: float
+      - name: fnumber2
+        summary: float option
+        type: float
+        aliases: ["f"]
       - name: date
         type: string
         default: today
