@@ -3,7 +3,7 @@ use strict;
 use warnings;
 package App::Spec::Completion::Bash;
 
-our $VERSION = '0.005_001'; # TRIAL VERSION
+our $VERSION = '0.008_001'; # TRIAL VERSION
 
 use Moo;
 extends 'App::Spec::Completion';
@@ -32,14 +32,14 @@ _$appname() \{
 
     COMPREPLY=()
     local program=$appname
-    local cur=\$\{COMP_WORDS[\$COMP_CWORD]\}
-#    echo "COMP_CWORD:\$COMP_CWORD cur:\$cur" >>/tmp/comp
+    local cur prev words cword
+    _init_completion -n : || return
     declare -a FLAGS
     declare -a OPTIONS
     declare -a MYWORDS
 
-    local INDEX=`expr \$COMP_CWORD - 1`
-    MYWORDS=("\$\{COMP_WORDS[@]:1:\$COMP_CWORD\}")
+    local INDEX=`expr \$cword - 1`
+    MYWORDS=("\$\{words[@]:1:\$cword\}")
 
     FLAGS=($flags_string)
     OPTIONS=($options_string)
@@ -49,12 +49,15 @@ $completion_outer
 \}
 
 _${appname}_compreply() \{
-    IFS=\$'\\n' COMPREPLY=(\$(compgen -W "\$1" -- \$\{COMP_WORDS\[COMP_CWORD\]\}))
+    local prefix=""
+    cur="\$(printf '%q' "\$cur")"
+    IFS=\$'\\n' COMPREPLY=(\$(compgen -P "\$prefix" -W "\$*" -- "\$cur"))
+    __ltrim_colon_completions "\$prefix\$cur"
 
     # http://stackoverflow.com/questions/7267185/bash-autocompletion-add-description-for-possible-completions
     if [[ \$\{#COMPREPLY[*]\} -eq 1 ]]; then # Only one completion
-        COMPREPLY=( \$\{COMPREPLY[0]%% -- *\} ) # Remove ' -- ' and everything after
-        COMPREPLY=( \$\{COMPREPLY[0]%% *\} ) # Remove trailing spaces
+        COMPREPLY=( "\$\{COMPREPLY[0]%% -- *\}" ) # Remove ' -- ' and everything after
+        COMPREPLY=( "\$\{COMPREPLY[0]%%+( )\}" ) # Remove trailing spaces
     fi
 \}
 
@@ -310,15 +313,16 @@ sub completion_options  {
 ${indent}  @{[ join '|', @option_strings ]})
 EOM
         if ($enum) {
-            my @list = map { s/ /\\ /g; $_ } @$enum;
-            local $" = q{"$'\\n'"};
+            my @list = @$enum;
             for (@list) {
                 s/['`]/'"'"'/g;
+                s/\\/\\\\/g;
+                s/ /\\\\\\\\ /g;
                 s/\$/\\\$/g;
-                $_ = "'$_'";
+                $_ = qq{"$_"};
             }
             $comp_value .= <<"EOM";
-${indent}    _${appname}_compreply "@list"
+${indent}    _${appname}_compreply @list
 ${indent}    return
 EOM
         }
@@ -379,7 +383,7 @@ sub dynamic_completion {
         $function = <<"EOM";
 $function_name() \{
     local __dynamic_completion
-    __dynamic_completion=`PERL5_APPSPECRUN_SHELL=bash PERL5_APPSPECRUN_COMPLETION_PARAMETER='$name' \${COMP_WORDS[@]}`
+    __dynamic_completion=\$(PERL5_APPSPECRUN_SHELL=bash PERL5_APPSPECRUN_COMPLETION_PARAMETER='$name' \${words[@]})
     __${appname}_dynamic_comp '$name' "\$__dynamic_completion"
 \}
 EOM
@@ -400,7 +404,7 @@ EOM
                         my @repl = @$replace;
                         if ($replace->[0] eq 'SHELL_WORDS') {
                             my $num = $replace->[1];
-                            my $index = "\$COMP_CWORD";
+                            my $index = "\$cword";
                             if ($num ne 'CURRENT') {
                                 if ($num =~ m/^-/) {
                                     $index .= $num;
@@ -409,7 +413,7 @@ EOM
                                     $index = $num - 1;
                                 }
                             }
-                            my $string = qq{"\$\{COMP_WORDS\[$index\]\}"};
+                            my $string = qq{"\$\{words\[$index\]\}"};
                             push @args, $string;
                         }
                     }
@@ -430,7 +434,8 @@ EOM
         chomp $string;
         $function = <<"EOM";
 $function_name() \{
-    local param_$shell_name=`$string`
+    local CURRENT_WORD="\${words\[\$cword\]\}"
+    local param_$shell_name="\$($string)"
     _${appname}_compreply "\$param_$shell_name"
 \}
 EOM
@@ -483,13 +488,15 @@ sub completion_parameter {
     my $enum = $param->enum;
     if ($enum) {
         my @list = @$enum;
-        local $" = q{"$'\\n'"};
         for (@list) {
             s/['`]/'"'"'/g;
+            s/\\/\\\\/g;
+            s/ /\\\\ /g;
             s/\$/\\\$/g;
+            $_ = qq{"$_"};
         }
         $comp = <<"EOM";
-${indent}    _${appname}_compreply "@list"
+${indent}    _${appname}_compreply @list
 EOM
     }
     elsif ($type eq "file" or $type eq "dir") {
@@ -514,7 +521,8 @@ sub _functions {
 __APPNAME_dynamic_comp() {
     local argname="$1"
     local arg="$2"
-    local comp name desc cols desclength formatted
+    local name desc cols desclength formatted
+    local comp=()
     local max=0
 
     while read -r line; do
@@ -537,12 +545,12 @@ __APPNAME_dynamic_comp() {
             [[ -z $cols ]] && cols=80
             desclength=`expr $cols - 4 - $max`
             formatted=`printf "%-*s -- %-*s" "$max" "$name" "$desclength" "$desc"`
-            comp="$comp$formatted"$'\n'
+            comp+=("$formatted")
         else
-            comp="$comp'$name'"$'\n'
+            comp+=("'$name'")
         fi
     done <<< "$arg"
-    _APPNAME_compreply "$comp"
+    _APPNAME_compreply ${comp[@]}
 }
 
 function __APPNAME_handle_options() {
