@@ -3,7 +3,7 @@ use strict;
 use warnings;
 package App::Spec::Completion::Zsh;
 
-our $VERSION = '0.005_001'; # TRIAL VERSION
+our $VERSION = '0.008_001'; # TRIAL VERSION
 
 use Moo;
 extends 'App::Spec::Completion';
@@ -265,8 +265,8 @@ sub dynamic_completion {
         $function = <<"EOM";
 $function_name() \{
     local __dynamic_completion
-    __dynamic_completion=`PERL5_APPSPECRUN_SHELL=zsh PERL5_APPSPECRUN_COMPLETION_PARAMETER='$name' "\${words[@]}"`
-    __${appname}_dynamic_comp '$name' "\$__dynamic_completion"
+    __dynamic_completion=\$(PERL5_APPSPECRUN_SHELL=zsh PERL5_APPSPECRUN_COMPLETION_PARAMETER='$name' "\${words[@]}")
+    __${appname}_dynamic_comp '$name' \$__dynamic_completion
 \}
 EOM
     }
@@ -315,7 +315,8 @@ EOM
         $function = <<"EOM";
 $function_name() \{
     local __dynamic_completion
-    IFS=\$'\\n' set -A __dynamic_completion `$string`
+    local CURRENT_WORD="\$words\[CURRENT\]"
+    IFS=\$'\\n' set -A __dynamic_completion \$( $string )
     compadd -X "$shell_name:" \$__dynamic_completion
 \}
 EOM
@@ -396,7 +397,11 @@ sub options {
             $comp .= $indent . ";;\n";
         }
         elsif ($enum) {
-            my @list = map { qq{"$_"} } @$enum;
+            my @list = map {
+                my $item = $_;
+                $item =~ s/:/\\:/g;
+                qq{"$item"};
+            } @$enum;
             $values = ":$name:(@list)";
         }
         elsif ($type eq "file" or $type eq "dir") {
